@@ -15,7 +15,7 @@ from PIL import Image
 import gi
 gi.require_version("Gtk", "3.0")
 gi.require_version("AccountsService", "1.0")
-from gi.repository import Gtk, GObject, Gio, GdkPixbuf, AccountsService
+from gi.repository import Gtk, GObject, Gio, GdkPixbuf, AccountsService, GLib
 
 gettext.install("cinnamon", "/usr/share/locale")
 
@@ -23,8 +23,8 @@ gettext.install("cinnamon", "/usr/share/locale")
 (INDEX_GID, INDEX_GROUPNAME) = range(2)
 
 class GroupDialog (Gtk.Dialog):
-    def __init__ (self, label, value):
-        super(GroupDialog, self).__init__()
+    def __init__ (self, label, value, parent = None):
+        super(GroupDialog, self).__init__(None, parent)
 
         try:
             self.set_modal(True)
@@ -93,7 +93,7 @@ class EditableEntry (Gtk.Notebook):
 
     __gsignals__ = {
         'changed': (GObject.SIGNAL_RUN_FIRST, None,
-                      (str,))
+                    (str,))
     }
 
     PAGE_BUTTON = 0
@@ -153,8 +153,8 @@ class EditableEntry (Gtk.Notebook):
 
 class PasswordDialog(Gtk.Dialog):
 
-    def __init__ (self, user, password_mask):
-        super(PasswordDialog, self).__init__()
+    def __init__ (self, user, password_mask, parent = None):
+        super(PasswordDialog, self).__init__(None, parent)
 
         self.user = user
         self.password_mask = password_mask
@@ -315,8 +315,8 @@ class PasswordDialog(Gtk.Dialog):
 
 class NewUserDialog(Gtk.Dialog):
 
-    def __init__ (self):
-        super(NewUserDialog, self).__init__()
+    def __init__ (self, parent = None):
+        super(NewUserDialog, self).__init__(None, parent)
 
         try:
             self.set_modal(True)
@@ -372,8 +372,8 @@ class NewUserDialog(Gtk.Dialog):
 
 class GroupsDialog(Gtk.Dialog):
 
-    def __init__ (self, username):
-        super(GroupsDialog, self).__init__()
+    def __init__ (self, username, parent = None):
+        super(GroupsDialog, self).__init__(None, parent)
 
         try:
             self.set_modal(True)
@@ -472,6 +472,7 @@ class Module:
 
             self.face_button = Gtk.Button()
             self.face_image = Gtk.Image()
+            self.face_image.set_size_request(96, 96)
             self.face_button.set_image(self.face_image)
             self.face_image.set_from_file("/usr/share/cinnamon/faces/user-generic.png")
             self.face_button.set_alignment(0.0, 0.5)
@@ -561,14 +562,14 @@ class Module:
         model, treeiter = self.users_treeview.get_selection().get_selected()
         if treeiter != None:
             user = model[treeiter][INDEX_USER_OBJECT]
-            dialog = PasswordDialog(user, self.password_mask)
+            dialog = PasswordDialog(user, self.password_mask, self.window)
             response = dialog.run()
 
     def _on_groups_button_clicked(self, widget):
         model, treeiter = self.users_treeview.get_selection().get_selected()
         if treeiter != None:
             user = model[treeiter][INDEX_USER_OBJECT]
-            dialog = GroupsDialog(user.get_user_name())
+            dialog = GroupsDialog(user.get_user_name(), self.window)
             response = dialog.run()
             if response == Gtk.ResponseType.OK:
                 groups = dialog.get_selected_groups()
@@ -611,30 +612,26 @@ class Module:
             filter.add_mime_type("image/*")
             dialog.add_filter(filter)
 
-            preview = Gtk.Image()
-            dialog.set_preview_widget(preview);
-            dialog.connect("update-preview", self.update_preview_cb, preview)
+            box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+            self.frame = Gtk.Frame(visible=False, no_show_all=True)
+            preview = Gtk.Image(visible=True)
+
+            box.pack_start(self.frame, False, False, 0)
+            self.frame.add(preview)
+            dialog.set_preview_widget(box)
+            dialog.set_preview_widget_active(True)
             dialog.set_use_preview_label(False)
 
+            box.set_margin_end(12)
+            box.set_margin_top(12)
+            box.set_size_request(128, -1)
+
+            dialog.connect("update-preview", self.update_preview_cb, preview)
+
             response = dialog.run()
             if response == Gtk.ResponseType.OK:
                 path = dialog.get_filename()
                 image = PIL.Image.open(path)
-                width, height = image.size
-                if width > height:
-                    new_width = height
-                    new_height = height
-                elif height > width:
-                    new_width = width
-                    new_height = width
-                else:
-                    new_width = width
-                    new_height = height
-                left = (width - new_width)/2
-                top = (height - new_height)/2
-                right = (width + new_width)/2
-                bottom = (height + new_height)/2
-                image = image.crop((left, top, right, bottom))
                 image.thumbnail((96, 96), Image.ANTIALIAS)
                 face_path = os.path.join(user.get_home_dir(), ".face")
                 image.save(face_path, "png")
@@ -646,15 +643,22 @@ class Module:
             dialog.destroy()
 
     def update_preview_cb (self, dialog, preview):
+        # Different widths make the dialog look really crappy as it resizes -
+        # constrain the width and adjust the height to keep perspective.
         filename = dialog.get_preview_filename()
-        if filename is None:
-            return
-        dialog.set_preview_widget_active(False)
-        if os.path.isfile(filename):
-            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(filename, 128, 128)
-            if pixbuf is not None:
-                preview.set_from_pixbuf (pixbuf)
-                dialog.set_preview_widget_active(True)
+        if filename is not None:
+            if os.path.isfile(filename):
+                try:
+                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(filename, 128, 128)
+                    if pixbuf is not None:
+                        preview.set_from_pixbuf(pixbuf)
+                        self.frame.show()
+                        return
+                except GLib.Error as e:
+                    print("Unable to generate preview for file '%s' - %s\n" % (filename, e.message))
+
+        preview.clear()
+        self.frame.hide()
 
     def _on_face_menuitem_activated(self, menuitem, path):
         if os.path.exists(path):
@@ -741,9 +745,30 @@ class Module:
             else:
                 self.account_type_combo.set_active(0)
 
-            if os.path.exists(user.get_icon_file()):
-                self.face_image.set_from_file(user.get_icon_file())
+            pixbuf = None
+            path = user.get_icon_file()
+            message = ""
+
+            if os.path.exists(path):
+                try:
+                    pixbuf = GdkPixbuf.Pixbuf.new_from_file(path)
+                except GLib.Error as e:
+                    message = "Could not load pixbuf from '%s': %s" % (path, e.message)
+                    error = True
+
+                if pixbuf != None:
+                    if pixbuf.get_height() > 96 or pixbuf.get_width() > 96:
+                        try:
+                            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(path, 96, 96)
+                        except GLib.Error as e:
+                            message = "Could not scale pixbuf from '%s': %s" % (path, e.message)
+                            error = True
+
+            if pixbuf:
+                self.face_image.set_from_pixbuf(pixbuf)
             else:
+                if message != "":
+                    print message
                 self.face_image.set_from_file("/usr/share/cinnamon/faces/user-generic.png")
 
             groups = []
@@ -777,10 +802,10 @@ class Module:
             user = model[treeiter][INDEX_USER_OBJECT]
             message = _("Are you sure you want to permanently delete %s and all the files associated with this user?") % user.get_user_name()
             d = Gtk.MessageDialog(self.window,
-                              Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                              Gtk.MessageType.QUESTION,
-                              Gtk.ButtonsType.YES_NO,
-                              message)
+                                  Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                                  Gtk.MessageType.QUESTION,
+                                  Gtk.ButtonsType.YES_NO,
+                                  message)
             d.set_markup(message)
             d.set_default_response(Gtk.ResponseType.NO)
             r = d.run()
@@ -792,7 +817,7 @@ class Module:
                     self.load_groups()
 
     def on_user_addition(self, event):
-        dialog = NewUserDialog()
+        dialog = NewUserDialog(self.window)
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
             if dialog.account_type_combo.get_active() == 1:
@@ -847,10 +872,10 @@ class Module:
             group = model[treeiter][INDEX_GROUPNAME]
             message = _("Are you sure you want to permanently delete %s?") % group
             d = Gtk.MessageDialog(self.window,
-                              Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                              Gtk.MessageType.QUESTION,
-                              Gtk.ButtonsType.YES_NO,
-                              message)
+                                  Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                                  Gtk.MessageType.QUESTION,
+                                  Gtk.ButtonsType.YES_NO,
+                                  message)
             d.set_markup(message)
             d.set_default_response(Gtk.ResponseType.NO)
             r = d.run()
@@ -860,7 +885,7 @@ class Module:
             d.destroy()
 
     def on_group_addition(self, event):
-        dialog = GroupDialog(_("Group Name"), "")
+        dialog = GroupDialog(_("Group Name"), "", self.window)
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
             subprocess.call(["groupadd", dialog.entry.get_text().lower()])
@@ -871,7 +896,7 @@ class Module:
         model, treeiter = self.groups_treeview.get_selection().get_selected()
         if treeiter != None:
             group = model[treeiter][INDEX_GROUPNAME]
-            dialog = GroupDialog(_("Group Name"), group)
+            dialog = GroupDialog(_("Group Name"), group, self.window)
             response = dialog.run()
             if response == Gtk.ResponseType.OK:
                 subprocess.call(["groupmod", group, "-n", dialog.entry.get_text().lower()])
@@ -882,7 +907,3 @@ class Module:
 if __name__ == "__main__":
     module = Module()
     Gtk.main()
-
-
-
-
